import numpy as np


class Higham(Exception):
    pass


def dual(x, p=None):
    """
    DUAL    Dual vector with respect to Holder p-norm.
        y = dual(x, p), where 1 <= p <= inf, is a vector of unit q-norm
        that is dual to X with respect to the p-norm, that is,
        norm(Y, q) = 1 where 1/p + 1/q = 1 and there is
        equality in the Holder inequality: X'*Y = norm(X, p)*norm(Y, q).
        Special case: DUAL(X), where X >= 1 is a scalar, returns Y such
                      that 1/X + 1/Y = 1.

        Called by PNORM.
    """

    if p is None:
        if len(x) == 1:
            y = 1 / (1 - 1 / x)
            return
        else:
            raise Higham('Second argument missing.')

    q = 1 / (1 - 1 / float(p))

    if np.linalg.norm(x, np.inf) == 0:
        y = x
        return

    if p == 1:
        # we want zero to be thought of as "positive"
        y = np.where(np.sign(x) == 0, 1, np.sign(x))

    elif p == np.inf:
        # y is a multiple of unit vector e_k.
        y = np.where(np.abs(x) == np.abs(x).max, np.sign(x), 0)

    else:
        # 1 < p < inf.  Dual is unique in this case.
        # This scaling helps to avoid under/over-flow.
        x = x / np.linalg.norm(x, np.inf)
        y = np.abs(x) ** (p - 1) * np.where(np.sign(x) == 0, 1, np.sign(x))
        y = y / np.linalg.norm(y, q)         # Normalize to unit q-norm.

    return y



import sys
if sys.version_info.major < 3 and sys.version_info.micro < 15:
    from distribute_setup import use_setuptools
    use_setuptools()

with open("README.md", "r") as fh:
    long_description = fh.read()
    
from setuptools import setup, find_packages

setup(name             = 'rogues',
      version          = '1.0.0',
      test_suite       = 'nose.collector',
      packages         = find_packages(),
      install_requires = ['numpy', 'scipy'],
      author           = 'Don MacMillen',
      author_email     = 'don@macmillen.net',
      url              = 'https://github.com/macd/rogues',
      description      = "Python and numpy port of Nicholas Higham's m*lab test matrices",
      long_description = long_description,
      long_description_content_type = "text/markdown",
      license          = 'MIT',
      keywords         = 'numpy scipy matplotlib linalg',
      zip_safe         = True,
      classifiers=[
                     'Development Status :: 4 - Beta',
                     'Intended Audience :: Developers',
                     'Intended Audience :: Science/Research',
                     'License :: OSI Approved :: MIT License',
                     'Operating System :: OS Independent',
                     'Topic :: Scientific/Engineering :: Mathematics',
      ],
)



"""
The "utils" module is a reimplmentation of a few of Nick Higham's "Matrix
Computation Toolbox" [1], into Python, Numpy, and Scipy.  They were
generally ported using the iPython shell and probably work best there.
They were developed from version 1.2 (released on 5-Sep-2002 and downloaded
17 Feb 2009.)

Also included in this package some are the generic matrix manipulation
functions that were inclued in Version 3, (1995 downloaded Feb 2009)
of Test Matrix package [2]

     [1] "N. J. Higham. The Matrix Computation Toolbox.
           http://www.ma.man.ac.uk/~higham/mctoolbox"
     [2] "N.J. Higham, The Test Matrix Toolbox for Matlab
           (Version 3.0) , NA Report 276, September 1995.
           http://www.maths.manchester.ac.uk/~higham/papers/misc.php "
     [3] "N.J. Higham, Algorithm 694: A collection of test matrices
           in MATLAB, ACM Trans. Math. Soft., 17(3):289--305, Sept. 1991.

Don MacMillen 15 May 2009
"""

from rogues.utils.augment import *
from rogues.utils.bandred import *
from rogues.utils.cgs import *
from rogues.utils.cond import *
from rogues.utils.condeig import *
from rogues.utils.cpltaxes import *
from rogues.utils.dual import *
from rogues.utils.fv import *
from rogues.utils.ge import *
from rogues.utils.gersh import *
# duplicate, don't use this one
# from rogues.utils.hankel import *
from rogues.utils.house import *
from rogues.utils.mdsmax import *
from rogues.utils.mgs import *
from rogues.utils.pow2 import *
from rogues.utils.ps import *
from rogues.utils.pscont import *
from rogues.utils.repmat import *
from rogues.utils.rq import *
from rogues.utils.skewpart import *
from rogues.utils.sparsify import *
from rogues.utils.sub import *
from rogues.utils.symmpart import *
from rogues.utils.toeplitz import *
from rogues.utils.treshape import *
from rogues.utils.tridiag import *
from rogues.utils.vand import *
from rogues.utils.vecperm import *



import numpy as np
import rogues


def dorr(n, theta=0.01, r_matrix=False, set_sparse=False):
    """
    dorr  dorr matrix - diagonally dominant, ill conditioned, tridiagonal.
      c, d, e = dorr(n, theta) returns the vectors defining a row diagonally
      dominant, tridiagonal m-matrix that is ill conditioned for small
      values of the parameter theta >= 0.
      If r_matrix is set to True, then c = full(tridiag(c,d,e)), i.e.,
      the matrix iself is returned.  The format for the matrix is set by
      the boolean set_sparse.  If set to true, return a matrix in sparse
      matrix format.  Default is False.

      The columns of inv(c) vary greatly in norm.  theta defaults to 0.01.
      The amount of diagonal dominance is given by (ignoring rounding errors):
            comp(c)*ones(n,1) = theta*(n+1)^2 * [1 0 0 ... 0 1]'.

      Reference:
      F.W. Dorr, An example of ill-conditioning in the numerical
      solution of singular perturbation problems, Math. Comp., 25 (1971),
      pp. 271-283.
    """

    c = np.zeros(n)
    e = np.zeros(n)
    d = np.zeros(n)

    # All length n for convenience.  Make c, e of length n-1 later.

    h = 1. / (n + 1)
    m = (n + 1) // 2
    term = theta / h ** 2

    i = slice(0, m)
    c[i] = -term * np.ones(m)
    e[i] = c[i] - (0.5 - np.arange(1, m + 1) * h) / h
    d[i] = -(c[i] + e[i])

    i = slice(m, n)
    e[i] = -term * np.ones(n - m)
    c[i] = e[i] + (0.5 - np.arange(1, m + 1) * h) / h
    d[i] = -(c[i] + e[i])

    # shorten
    c = c[1:n]
    e = e[0:n - 1]

    if r_matrix:
        if set_sparse:
            c = rogues.tridiag(c, d, e)
        else:
            c = np.diag(c, -1) + np.diag(d) + np.diag(e, 1)
        # if we are to return the matrix, return _only_ the matrix
        return c

    # Return the diagonals of the matrix
    return c, d, e



import numpy as np


class Higham(Exception):
    pass


def triw(n, alpha=-1, k=0):
    """
    triw   Upper triangular matrix discussed by Wilkinson and others.
       triw(n, alpha, k) is the upper triangular matrix with ones on
       the diagonal and ALPHAs on the first K >= 0 superdiagonals.
       n may be a 2-tuple, in which case the matrix is n[0]-by-n[1] and
       upper trapezoidal.
       Defaults: alpha = -1,
                 k = n - 1     (full upper triangle).
       triw(n) is a matrix discussed by Kahan, Golub and Wilkinson.

       Ostrowski (1954) shows that
         COND(TRIW(N,2)) = COT(PI/(4*N))^2,
       and for large ABS(ALPHA),
         COND(TRIW(N,ALPHA)) is approximately ABS(ALPHA)^N*SIN(PI/(4*N-2)).

       Adding -2^(2-N) to the (N,1) element makes TRIW(N) singular,
       as does adding -2^(1-N) to all elements in the first column.

       References:
       G.H. Golub and J.H. Wilkinson, Ill-conditioned eigensystems and the
          computation of the Jordan canonical form, SIAM Review,
          18(4), 1976, pp. 578-619.
       W. Kahan, Numerical linear algebra, Canadian Math. Bulletin,
          9 (1966), pp. 757-801.
       A.M. Ostrowski, On the spectrum of a one-parametric family of
          matrices, J. Reine Angew. Math., 193 (3/4), 1954, pp. 143-160.
       J.H. Wilkinson, Singular-value decomposition---basic aspects,
          in D.A.H. Jacobs, ed., Numerical Software---Needs and Availability,
          Academic Press, London, 1978, pp. 109-135.
    """
    try:
        m, n = n
    except TypeError:
        m = n

    if k == 0:
        k = n - 1

    if np.array(alpha).size != 1:
        raise Higham("Second Argument Must Be A Scalar.")

    t = np.tril(np.eye(m, n) + alpha * np.triu(np.ones((m, n)), 1), k)

    return t



import numpy as np


class Higham(Exception):
    pass


def hanowa(n, d=1):
    """
    hanowa  a matrix whose eigenvalues lie on a vertical line in the
            complex plane.
        hanowa(n, d) is the n-by-n block 2x2 matrix (thus n = 2m must be even)
                      [d*np.eye(m)   -np.diag(np.range(1,m+1))
                       np.diag(arange(1,m+1))   d*np.eye(m) ]
        it has complex eigenvalues lambda(k) = d +/- k*i  (1 <= k <= m).
        parameter d defaults to -1.

        Reference:
        E. Hairer, S.P. Norsett and G. Wanner, Solving Ordinary
        Differential Equations I: Nonstiff Problems, Springer-Verlag,
        Berlin, 1987. (pp. 86-87)
    """
    if n % 2:
        raise Higham('n must be even.')
    else:
        m = n // 2

    dg = np.diag(np.arange(1, m + 1))
    # Style / lint checkers will complain about the following two variables
    # not being used because we are passing them into np.bmat as a string.
    mdg = -1 * dg
    de = d * np.eye(m)
    # Using this string form raises a deprecation error in numpy 2.1.0 and python 3.12.5
    a = np.bmat('de,  mdg; dg, de')

    return a



import numpy as np


def sub(a, i, j=None):
    """
    SUB     Principal submatrix.
        sub(a,i,j) is a[i:j, i:j].
        sub(a,i)  is the leading principal submatrix of order i,
        a[0:i, 0:i), if i > 0, and the trailing principal submatrix
        of order ABS(i) if i<0.
    """
    # editorial comment: this function seems unnecessary... it strikes
    # me that you would always want the bare code rather than the function
    # call because it is self evident what it does
    if j is None:
        if i >= 0:
            s = a[0:i, 0:i]
        else:
            n = np.min(a.shape)
            s = a[n + i:n, n + i:n]

    else:
        s = a[i:j, i:j]

    return s



import numpy as np


def lehmer(n):
    """
    lehmer  lehmer matrix - symmetric positive definite.
        a = lehmer(n) is the symmetric positive definite n-by-n matrix with
                         a[i,j] = (i+1)/(j+1) for j >= i.
        a is totally nonnegative.  inv(a) is tridiagonal, and explicit
        formulas are known for its entries.
        n <= cond(a) <= 4*n*n.

        References:
        M. Newman and J. Todd, The evaluation of matrix inversion
           programs, J. Soc. Indust. Appl. Math., 6 (1958), pp. 466-476.
        Solutions to problem E710 (proposed by D.H. Lehmer): The inverse
           of a matrix, Amer. Math. Monthly, 53 (1946), pp. 534-535.
        J. Todd, Basic Numerical Mathematics, Vol. 2: Numerical Algebra,
           Birkhauser, Basel, and Academic Press, New York, 1977, p. 154.
    """
    a = np.outer(np.ones(n), np.arange(1, n + 1))
    a = a / a.T
    a = np.tril(a) + np.tril(a, -1).T

    return a



import numpy as np


class Higham(Exception):
    pass


def cond(a, p=2):
    """
    COND   Matrix condition number in 1, 2, Frobenius, or infinity norm.
       For p = 1, 2, 'fro', inf,  COND(A,p) = NORM(A,p) * NORM(INV(A),p).
       If p is omitted then p = 2 is used.
       A may be a rectangular matrix if p = 2; in this case COND(A)
       is the ratio of the largest singular value of A to the smallest
       (and hence is infinite if A is rank deficient).

       See also RCOND, NORM, CONDEST, NORMEST.

       This replicates (essentially) np.linalg.cond
    """

    if len(a) == 0:  # Handle null matrix.
        y = np.NaN
        return

    m, n = a.shape
    if m != n and p != 2:
        raise Higham('a is rectangular.  Use the 2 norm.')

    if p == 2:
        u, s, v = np.linalg.svd(a)
        if (s == 0).any():   # Handle singular matrix
            print('Condition is infinite')
            y = np.Inf
        y = max(s) / min(s)
    else:
        #  We'll let NORM pick up any invalid p argument.
        y = np.linalg.norm(a, p) * np.linalg.norm(np.linalg.inv(a), p)

    return y



import numpy as np
from rogues.utils import toeplitz


def chow(n, alpha=1, delta=0):
    """
    chow(n, alpha, delta):  chow matrix - a singular toeplitz
        lower hessenberg matrix.
        a = chow(n, alpha, delta) is a toeplitz lower hessenberg matrix
        a = h(alpha) + delta*eye, where h(i,j) = alpha^(i-j+1).
        h(alpha) has p = floor(n/2) zero eigenvalues, the rest being
        4*alpha*cos( k*pi/(n+2) )^2, k=1:n-p.
        defaults: alpha = 1, delta = 0.

        References:
        T.S. Chow, A class of Hessenberg matrices with known
           eigenvalues and inverses, SIAM Review, 11 (1969), pp. 391-395.
        G. Fairweather, On the eigenvalues and eigenvectors of a class of
           Hessenberg matrices, SIAM Review, 13 (1971), pp. 220-221.
    """
    a = toeplitz(alpha ** np.arange(1, n + 1),
                 np.hstack((alpha, 1, np.zeros(n - 2)))) + delta * np.eye(n)
    return a



import numpy as np


def cycol(mn, k=None):
    """
    cycol   matrix whose columns repeat cyclically.
        a = cycol(mn, k) (mn is a 2-tuple) is an m-by-n matrix of
        the form a = b(1:m,1:n) where b = [c c c...] and c = randn(m, k).
        Thus a's columns repeat cyclically, and a has rank at most k.
        k need not divide n. k defaults to round(n/4).
        cycol(n, k), where n is a scalar, is the same as cycol((n n), k).

        This type of matrix can lead to underflow problems for Gaussian
        elimination: see NA Digest Volume 89, Issue 3 (January 22, 1989).
    """
    try:
        m, n = mn
    except (TypeError, AttributeError):
        m = mn
        n = mn

    if k is None:
        k = max(n // 4, 1)
    else:
        if k > n:
            raise ValueError("k cannot be greater than the "
                             "max matrix dimension")

    a = np.random.randn(m, k)

    for i in range(1, int(np.ceil(n / k))):
        a = np.hstack((a, a[:, 0:k]))

    # Truncate matrix down to desired size if we concat'ed too much
    a = a[:, 0:n]

    return a



import numpy as np


def pascal(n, k=0):
    """
    PASCAL  Pascal matrix.
        p = pascal(n) is the pascal matrix of order n: a symmetric positive
        definite matrix with integer entries taken from Pascal's
        triangle.
        The Pascal matrix is totally positive and its inverse has
        integer entries.  Its eigenvalues occur in reciprocal pairs.
        cond(p) is approximately 16**n/(n*pi) for large n.
        pascal(n,1) is the lower triangular cholesky factor (up to signs
        of columns) of the pascal matrix.   It is involutary (is its own
        inverse).
        pascal(n,2) is a transposed and permuted version of pascal(n,1)
        which is a cube root of the identity.

        References:
        R. Brawer and M. Pirovino, The linear algebra of the Pascal matrix,
           Linear Algebra and Appl., 174 (1992), pp. 13-23 (this paper
           gives a factorization of L = PASCAL(N,1) and a formula for the
           elements of L^k).
        N.J. Higham, Accuracy and Stability of Numerical Algorithms,
           Society for Industrial and Applied Mathematics, Philadelphia, PA,
           USA, 2002; sec. 28.4.
        S. Karlin, Total Positivity, Volume 1, Stanford University Press,
           1968.  (Page 137: shows i+j-1 choose j is TP (i,j=0,1,...).
                   PASCAL(N) is a submatrix of this matrix.)
        M. Newman and J. Todd, The evaluation of matrix inversion programs,
           J. Soc. Indust. Appl. Math., 6(4):466--476, 1958.
        H. Rutishauser, On test matrices, Programmation en Mathematiques
           Numeriques, Editions Centre Nat. Recherche Sci., Paris, 165,
           1966, pp. 349-365.  (Gives an integral formula for the
           elements of PASCAL(N).)
        J. Todd, Basic Numerical Mathematics, Vol. 2: Numerical Algebra,
           Birkhauser, Basel, and Academic Press, New York, 1977, p. 172.
        H.W. Turnbull, The Theory of Determinants, Matrices, and Invariants,
           Blackie, London and Glasgow, 1929.  (PASCAL(N,2) on page 332.)

    Note added in porting: for pascal(10) the properties above seem to be
    verified, especially the eigenvalues being reprocal pairs (all of which
    have the same constant value of one).  det(pascal(n)) is also ~one up to
    and including n=17.  At n=18 things go to hell and this appears to be
    where we step off a cliff
    """
    p = np.diag((-1) ** np.arange(n))
    p[:, 0] = np.ones(n)

    #  Generate the Pascal Cholesky factor (up to signs).
    for j in range(1, n - 1):
        for i in range(j + 1, n):
            p[i, j] = p[i - 1, j] - p[i - 1, j - 1]

    if k == 0:
        p = p @ p.T

    elif k == 2:
        p = np.rot90(p, 3)
        # CHECKME need to check .m file to see if this logic is correct
        if n / 2 == round(n / 2):
            p = -p

    return p



import numpy as np


class Higham(Exception):
    pass


def invhess(x, y=None):
    """
    invhess  inverse of an upper hessenberg matrix.
         invhess(x, y), where x is an n-vector and y an n-1 vector,
         is the matrix whose lower triangle agrees with that of
         np.outer(np.ones(n), x) and whose strict upper triangle agrees with
         that of np.outer(np.hstack((1, y)), np.ones(n))
         The matrix is nonsingular if x[1] != 0 and x[i+1] != y[i]
         for all i, and its inverse is an upper hessenberg matrix.
         If y is omitted it defaults to -x[0:n-2]
         Special case: if x is a scalar invhess(x) is the same as
         invhess(arange(1, x + 1)).

         References:
         F.N. Valvi and V.S. Geroyannis, Analytic inverses and
             determinants for a class of matrices, IMA Journal of Numerical
             Analysis, 7 (1987), pp. 123-128.
         W.-L. Cao and W.J. Stewart, A note on inverses of Hessenberg-like
             matrices, Linear Algebra and Appl., 76 (1986), pp. 233-240.
         Y. Ikebe, On inverses of Hessenberg matrices, Linear Algebra and
             Appl., 24 (1979), pp. 93-97.
         P. Rozsa, On the inverse of band matrices, Integral Equations and
             Operator Theory, 10 (1987), pp. 82-95.
    """
    try:
        n = np.max(x.shape)
    except AttributeError:
        n = x
        x = np.arange(1, n + 1)
        y = -1 * x

    a = np.outer(np.ones(n), x)
    for j in range(1, n):
        a[:j - 1, j] = y[:j - 1]

    return a



import scipy.sparse as sparse
from rogues.utils import tridiag


def poisson(n):
    """
    poisson   Block tridiagonal matrix from Poisson's equation (sparse).
          poisson(n) is the block tridiagonal matrix of order n**2
          resulting from discretizing Poisson's equation with the
          5-point operator on an n-by-n mesh.

          Reference:
          G.H. Golub and C.F. Van Loan, Matrix Computations, second edition,
          Johns Hopkins University Press, Baltimore, Maryland, 1989
          (Section 4.5.4).
    """
    s = tridiag(n, -1, 2, -1)
    i = sparse.eye(n, n)
    a = sparse.kron(i, s) + sparse.kron(s, i)

    return a



import numpy as np


def rando(n, k=1):
    """
    RANDO   Random matrix with elements -1, 0 or 1.
        a = rando(n, k) is a random n-by-n matrix with elements from
        one of the following discrete distributions (default k = 1):
          k = 1:  a[i,j] =  0 or 1    with equal probability,
          k = 2:  a[i,j] = -1 or 1    with equal probability,
          k = 3:  a[i,j] = -1, 0 or 1 with equal probability.
        n may be a 2-tuple, in which case the matrix is n[0]-by-n[1].
    """
    try:
        m, n = n                  # Parameter n specifies dimension: m-by-n.
    except TypeError:
        m = n

    if k == 1:                    # {0, 1}
        a = np.floor(np.random.rand(m, n) + .5)

    elif k == 2:                  # {-1, 1}
        a = 2 * np.floor(np.random.rand(m, n) + .5) - 1

    elif k == 3:                  # {-1, 0, 1}
        a = np.round(3 * np.random.rand(m, n) - 1.5)

    return a



import numpy as np


def frank(n, k=0):
    """
    frank(n, k = 0) Frank matrix---ill conditioned eigenvalues.
        f = frank(n, k) is the Frank matrix of order N.  It is upper
        Hessenberg with determinant 1.  K = 0 is the default; if K = 1 the
        elements are reflected about the anti-diagonal (1,N)--(N,1).
        F has all positive eigenvalues and they occur in reciprocal pairs
        (so that 1 is an eigenvalue if N is odd).
        The eigenvalues of F may be obtained in terms of the zeros of the
        Hermite polynomials.
        The FLOOR(N/2) smallest eigenvalues of F are ill conditioned,
        the more so for bigger N.

        DET(FRANK(N)') comes out far from 1 for large N---see Frank (1958)
        and Wilkinson (1960) for discussions.

        This version incorporates improvements suggested by W. Kahan.

        References:
        W.L. Frank, Computing eigenvalues of complex matrices by determinant
           evaluation and by methods of Danilewski and Wielandt, J. Soc.
           Indust. Appl. Math., 6 (1958), pp. 378-392 (see pp. 385, 388).
        G.H. Golub and J.H. Wilkinson, Ill-conditioned eigensystems and the
           computation of the Jordan canonical form, SIAM Review, 18 (1976),
             pp. 578-619 (Section 13).
        H. Rutishauser, On test matrices, Programmation en Mathematiques
           Numeriques, Editions Centre Nat. Recherche Sci., Paris, 165,
           1966, pp. 349-365.  Section 9.
        J.H. Wilkinson, Error analysis of floating-point computation,
           Numer. Math., 2 (1960), pp. 319-340 (Section 8).
        J.H. Wilkinson, The Algebraic Eigenvalue Problem, Oxford University
           Press, 1965 (pp. 92-93).
        The next two references give details of the eigensystem, as does
        Rutishauser (see above).
        P.J. Eberlein, A note on the matrices denoted by B_n, SIAM J. Appl.
           Math., 20 (1971), pp. 87-92.
        J.M. Varah, A generalization of the Frank matrix, SIAM J. Sci. Stat.
           Comput., 7 (1986), pp. 835-839.

    NOTE: This is a slightly different implementation from Version 3.  It
    loosely follows the TOMS694 version, _except_ that it directly computes
    the frank matrix, instead of its anti-transposed version.  Therefore,
    we take the anti-transpose if k == 1
    """

    f = np.ones((n, n)) * np.arange(n, 0, -1)
    f = np.minimum(f, f.T)
    #   take upper Hessenberg part.
    f = np.triu(f, -1)
    if k == 1:
        f = f[::-1, ::-1].T

    return f



import numpy as np
import rogues


def hadamard(n):
    """
    HADAMARD  Hadamard matrix.
          HADAMARD(N) is a Hadamard matrix of order N, that is,
          a matrix H with elements 1 or -1 such that H*H' = N*EYE(N).
          An N-by-N Hadamard matrix with N>2 exists only if REM(N,4) = 0.
          This function handles only the cases where N, N/12 or N/20
          is a power of 2.

          Reference:
          S.W. Golomb and L.D. Baumert, The search for Hadamard matrices,
             Amer. Math. Monthly, 70 (1963) pp. 12-17.
          http://en.wikipedia.org/wiki/Hadamard_matrix
          Weisstein, Eric W. "Hadamard Matrix." From MathWorld--
             A Wolfram Web Resource:
             http://mathworld.wolfram.com/HadamardMatrix.html
    """

    f, e = np.frexp(np.array([n, n / 12., n / 20.]))

    try:
        # If more than one condition is satified, this will always
        # pick the first one.
        k = [i for i in range(3) if (f == 0.5)[i] and (e > 0)[i]].pop()
    except IndexError:
        raise ValueError('N, N/12 or N/20 must be a power of 2.')

    e = e[k] - 1

    if k == 0:        # N = 1 * 2^e;
        h = np.array([1])

    elif k == 1:      # N = 12 * 2^e;
        tp = rogues.toeplitz(np.array([-1, -1, 1, -1, -1, -1, 1, 1, 1, -1, 1]),
                             np.array([-1, 1, -1, 1, 1, 1, -1, -1, -1, 1, -1]))
        h = np.vstack((np.ones((1, 12)), np.hstack((np.ones((11, 1)), tp))))

    elif k == 2:     # N = 20 * 2^e;
        hk = rogues.hankel(
                np.array([-1, -1, 1, 1, -1, -1, -1, -1, 1,
                          -1, 1, -1, 1, 1, 1, 1, -1, -1, 1]),
                np.array([1, -1, -1, 1, 1, -1, -1, -1, -1,
                          1, -1, 1, -1, 1, 1, 1, 1, -1, -1]))
        h = np.vstack((np.ones((1, 20)), np.hstack((np.ones((19, 1)), hk))))

    #  Kronecker product construction.

    mh = -1 * h
    for i in range(e):
        ht = np.hstack((h, h))
        hb = np.hstack((h, mh))
        h = np.vstack((ht, hb))
        mh = -1 * h

    return h
